" Vim syntax file
" Language:		Playstation2 VPU assembler [for ee-dvp-as]
" Maintainer:	Alex Mole
" Last Change:	2003 May
"

if version < 600
	syntax clear
elseif exists("b:current_syntax")
	finish
endif

syntax case ignore

" in order to match things like "lqi.xyz", we need to remove '.'
" from keyword searches, so we can define it as <opcode>[<dest>]
setlocal iskeyword-=.

syn region asmEEdvpString	start="\"" skip="\\\"" end="\"" oneline

syn match asmEEdvpLabel	"^[a-zA-Z_][0-9a-zA-Z_]*:"he=e-1
syn match asmEEdvpSymbol "[a-zA-Z_][a-zA-Z0-9_]*"

syn keyword asmEEdvpFlags	e
syn match asmEEdvpOperator	":+,()\*\\"
syn match asmEEdvpOperator	"\[[er]\]" contains=asmEEdvpFlags

syn keyword asmEEdvpRegister vi00 vi01 vi02 vi03 vi04 vi05 vi06 vi07
syn keyword asmEEdvpRegister vi08 vi09 vi10 vi11 vi12 vi13 vi14 vi15
syn keyword asmEEdvpRegister acc q i p r

syn match asmEEdvpRegister "vf[012][0-9][xyzw]*"
syn match asmEEdvpRegister "vf\(30\|31\)[xyzw]*"

syn match asmEEdvpDestination	"\.[xyzw]\{1,4\}"
syn match asmEEdvpDestination	")\s*[xyzw]\{1,4\}"hs=s+1

syn keyword asmEEdvpDirective	 unpack 
syn keyword asmEEdvpDirective	 mscal base offset mscnt
syn keyword asmEEdvpDirective	 dmacnt mpg 
syn keyword asmEEdvpDirective	 dmanext dmaend
syn match asmEEdvpDirective	"\.align\_s"
syn match asmEEdvpDirective	"\.dmapackvif\_s"
syn match asmEEdvpDirective	"\.else\_s"
syn match asmEEdvpDirective	"\.enddmadata\_s"
syn match asmEEdvpDirective	"\.endif\_s"
syn match asmEEdvpDirective	"\.endm\_s"
syn match asmEEdvpDirective	"\.endmpg\_s"
syn match asmEEdvpDirective	"\.endunpack\_s"
syn match asmEEdvpDirective	"\.global\_s"
syn match asmEEdvpDirective	"\.if\_s"
syn match asmEEdvpDirective	"\.include\_s"
syn match asmEEdvpDirective	"\.macro\_s"


" upper instructions
syn keyword asmEEdvpOpcodeU	abs opmula opmulb clip
syn keyword asmEEdvpOpcodeU	add addi addq addx addy addz addw
syn keyword asmEEdvpOpcodeU	adda addai addaq addax adday addaz addaw
syn keyword asmEEdvpOpcodeU	sub subi subq subx suby subz subw
syn keyword asmEEdvpOpcodeU	suba subai subaq subax subay subaz subaw
syn keyword asmEEdvpOpcodeU	mul muli mulq mulx muly mulz mulw
syn keyword asmEEdvpOpcodeU	mula mulai mulaq mulax mulay mulaz mulaw
syn keyword asmEEdvpOpcodeU	madd maddi maddq maddx maddy maddz maddw
syn keyword asmEEdvpOpcodeU	madda maddai maddaq maddax madday maddaz maddaw
syn keyword asmEEdvpOpcodeU	msub msubi msubq msubx msuby msubz msubw
syn keyword asmEEdvpOpcodeU	msuba msubai msubaq msubax msubay msubaz msubaw
syn keyword asmEEdvpOpcodeU	max maxi maxx maxy maxz maxw
syn keyword asmEEdvpOpcodeU	mini minii minix miniy miniz miniw
syn keyword asmEEdvpOpcodeU	itof0 itof4 itof12 itof15
syn keyword asmEEdvpOpcodeU	ftoi0 ftoi4 ftoi12 ftoi15
" lower instructions
syn keyword asmEEdvpOpcodeL	div sqrt rsqrt iadd iaddi iaddiu iand ior isub
syn keyword asmEEdvpOpcodeL	isubiu jr move mfir mtir mr32 lq lqd lqi sq sqi sqd
syn keyword asmEEdvpOpcodeL	ilw isw ilwr iswr loi rinit rget rnext rxor waitq
syn keyword asmEEdvpOpcodeL	fsand fseq fsor fsset fmand fmeq fmor fcand fceq 
syn keyword asmEEdvpOpcodeL	fcor fcset fcget ibeq ibgez ibgtz iblez ibltz ibne
syn keyword asmEEdvpOpcodeL	b bal mfp waitp esadd ersadd eleng erleng eatanxy
syn keyword asmEEdvpOpcodeL	eatanxz esum ercpr esqrt ersqrt esin eatan eexp
syn keyword asmEEdvpOpcodeL	xgkick xtop xitop

" make this an individual class so it can be a different colour
syn keyword asmEEdvpNOP		nop


syn match asmEEdvpType "\.float\_s"
syn match asmEEdvpType "\.int\_s"
syn match asmEEdvpType "\.short\_s"
syn match asmEEdvpType "\.byte\_s"
" include the macros defined in vumacros.h
syn keyword asmEEdvpType fx fxy fyx fxyz fzyx fxyzw fwzyx
syn keyword asmEEdvpType ix ixy iyx ixyz izyx ixyzw iwzyx
syn keyword asmEEdvpType sx sxy syx sxyz szyx sxyzw swzyx
syn keyword asmEEdvpType bx bxy byx bxyz bzyx bxyzw bwzyx

syn keyword asmEEdvpConstant	S_32 S_16 S_8
syn keyword asmEEdvpConstant	V2_32 V2_16 V2_8
syn keyword asmEEdvpConstant	V3_32 V3_16 V3_8
syn keyword asmEEdvpConstant	V4_32 V4_16 V4_8 V4_5

syn match asmEEdvpSection	"\.data\_s"
syn match asmEEdvpSection	"\.text\_s"
syn match asmEEdvpSection	"\.vu\_s"

" taken from asm68k.vim
syn match hexNumber		"0x[0-9a-fA-F]\+"
syn match decNumber		"\<[0-9]\+D\=\>"
syn match floatE		"_*E_*" contained
syn match floatExponent	"_*E_*[-+]\=[0-9]\+" contained contains=floatE
syn match floatNumber	"[-+]\=[0-9]\+_*E_*[-+]\=[0-9]\+" contains=floatExponent
syn match floatNumber	"[-+]\=[0-9]\+\.[0-9]\+\(E[-+]\=[0-9]\+\)\=" contains=floatExponent
syn match floatNumber	":\([0-9a-f]\+_*\)\+"

syn match asmEEdvpComment	";.*"hs=s
syn match asmEEdvpComment	"#.*"hs=s
syn region asmEEdvpComment	start="/\*" end="\*/"
syn match asmEEdvpCommentError	"\*/"



" Define the default highlighting.
" " For version 5.7 and earlier: only when not done already
" " For version 5.8 and later: only when an item doesn't have highlighting yet
if version >= 508 || !exists("did_asm_syntax_inits")
	if version < 508
		let did_asm_syntax_inits = 1
			command -nargs=+ HiLink hi link <args>
	else
		command -nargs=+ HiLink hi def link <args>
	endif
       

	" the default methods for highlighting
	HiLink asmEEdvpRegister		Identifier
	HiLink asmEEdvpOpcodeU		Statement
	HiLink asmEEdvpOpcodeL		Statement
	HiLink asmEEdvpFlags		Identifier
	HiLink asmEEdvpDestination	Statement
	HiLink asmEEdvpSection		Special
	HiLink asmEEdvpComment		Comment
	HiLink asmEEdvpCommentError	cCommentError
	HiLink asmEEdvpType			Type
	HiLink asmEEdvpDirective	Special
	HiLink asmEEdvpLabel		Identifier
	HiLink asmEEdvpSymbol		Identifier
	HiLink asmEEdvpString		String
	HiLink asmEEdvpConstant		Constant

	" taken from asm.vim
	HiLink asmInclude	Include
	HiLink asmCond		PreCondit
	HiLink asmMacro		Macro

	" taken from asm68k.vim
	HiLink hexNumber		Number		" Constant
	HiLink decNumber		Number		" Constant
	HiLink floatNumber		Number		" Constant
	HiLink floatExponent	Number		" Constant
	HiLink floatE			SpecialChar	" Statement
	"HiLink floatE		Number		" Constant

	"make NOPs a different colour
	hi asmEEdvpNOP	ctermfg=darkred

	" this is dubious... :)
	hi asmEEdvpRegister	ctermfg=darkgreen

	delcommand HiLink
endif

let b:current_syntax = "asmEEdvp"


